@@ -11,7 +11,7 @@ import copyreg as _copyreg
 from . import util as _util
 import unicodedata as _unicodedata
 from . import uniprops as _uniprops
-from typing import Generic, AnyStr, Optional, Match, Any, Pattern
+from typing import Generic, AnyStr, Match, Any, Pattern, cast
 
 if sys.version_info >= (3, 11):
     import re._parser as _parser  # type: ignore[import]
@@ -37,11 +37,19 @@ _STANDARD_ESCAPES = frozenset(('a', 'b', 'f', 'n', 'r', 't', 'v'))
 _CURLY_BRACKETS = frozenset(('{', '}'))
 _PROPERTY_STRIP = frozenset((' ', '-', '_'))
 _PROPERTY = _WORD | _DIGIT | _PROPERTY_STRIP
-_GLOBAL_FLAGS = frozenset(('a', 'u', 'L'))
-_SCOPED_FLAGS = frozenset(('i', 'm', 's', 'u', 'x'))
+_GLOBAL_FLAGS = frozenset(('a', 'i', 'L', 'm', 's', 'u', 'x'))
+_SCOPED_FLAGS = frozenset(('i', 'm', 's', 'x'))
+_SCOPED_END = frozenset((':', ')'))
 
 _CURLY_BRACKETS_ORD = frozenset((0x7b, 0x7d))
 
+_COMPATIBILITY_PROPERTIES = frozenset(
+    (
+        'alpha', 'lower', 'upper', 'punct', 'digit', 'xdigit', 'alnum',
+        'space', 'blank', 'cntrl', 'graph', 'print', 'word'
+    )
+)
+
 # Case upper or lower
 _UPPER = 1
 _LOWER = 2
@@ -77,7 +85,7 @@ class _SearchParser(Generic[AnyStr]):
     _re_start_wb = r"\b(?=\w)"
     _re_end_wb = r"\b(?<=\w)"
     _line_break = r'(?:\r\n|(?!\r\n)[\n\v\f\r\x85\u2028\u2029])'
-    _bytes_line_break = r'(?:\r\n|(?!\r\n)[\n\v\f\r\x85])'
+    _bytes_line_break = r'(?>\r\n|[\n\v\f\r\x85])' if _util.PY311 else r'(?:\r\n|(?!\r\n)[\n\v\f\r\x85])'
     _grapheme_cluster = r'(?:{}{}*(?!{}))'
 
     verbose: bool
@@ -88,7 +96,7 @@ class _SearchParser(Generic[AnyStr]):
     is_bytes: bool
     search: AnyStr
 
-    def __init__(self, search: AnyStr, re_verbose: bool = False, re_unicode: Optional[bool] = None) -> None:
+    def __init__(self, search: AnyStr, re_verbose: bool = False, re_unicode: bool | None = None) -> None:
         """Initialize."""
 
         if isinstance(search, bytes):
@@ -173,20 +181,24 @@ class _SearchParser(Generic[AnyStr]):
     def flags(self, text: str, scoped: bool = False) -> None:
         """Analyze flags."""
 
+        flags = text.split('-')
+        enable = flags[0]
+        disable = flags[1] if len(flags) > 1 else ''
+
         global_retry = False
-        if ('a' in text or 'L' in text) and self.unicode:
+        if ('a' in enable or 'L' in enable) and self.unicode:
             self.unicode = False
             if not scoped:
                 self.temp_global_flag_swap["unicode"] = True
                 global_retry = True
-        elif 'u' in text and not self.unicode and not self.is_bytes:
+        elif 'u' in enable and not self.unicode and not self.is_bytes:
             self.unicode = True
             if not scoped:
                 self.temp_global_flag_swap["unicode"] = True
                 global_retry = True
-        if '-x' in text and self.verbose:
+        if 'x' in disable and self.verbose:
             self.verbose = False
-        elif 'x' in text and not self.verbose:
+        elif 'x' in enable and not self.verbose:
             self.verbose = True
             if not scoped:
                 self.temp_global_flag_swap["verbose"] = True
@@ -204,8 +216,8 @@ class _SearchParser(Generic[AnyStr]):
             c = next(i)
             if c.upper() in _ASCII_LETTERS:
                 prop.append(c)
-            elif not brackets and c != '{' or brackets and c != ':':
-                raise SyntaxError("Unicode property missing '{{' at {}!".format(i.index - 1))
+            elif (not brackets and c != '{') or (brackets and c != ':'):
+                raise SyntaxError(f"Unicode property missing '{{' at {i.index - 1}!")
             else:
                 c = next(i)
                 if c == '^':
@@ -214,7 +226,7 @@ class _SearchParser(Generic[AnyStr]):
 
                 while c not in (':', '=', '}'):
                     if c not in _PROPERTY:
-                        raise SyntaxError('Invalid Unicode property character at {}!'.format(i.index - 1))
+                        raise SyntaxError(f'Invalid Unicode property character at {i.index - 1}!')
                     if c not in _PROPERTY_STRIP:
                         prop.append(c)
                     c = next(i)
@@ -236,24 +248,32 @@ class _SearchParser(Generic[AnyStr]):
                     if not skip:
                         while c != end:
                             if c not in _PROPERTY:
-                                raise SyntaxError('Invalid Unicode property character at {}!'.format(i.index - 1))
+                                raise SyntaxError(f'Invalid Unicode property character at {i.index - 1}!')
                             if c not in _PROPERTY_STRIP:
                                 value.append(c)
                             c = next(i)
                         if brackets and c == ':':
                             c = next(i)
                             if c != ']':
-                                raise SyntaxError('Invalid Unicode property character at {}!'.format(i.index - 1))
+                                raise SyntaxError(f'Invalid Unicode property character at {i.index - 1}!')
                         if not value:
                             raise SyntaxError('Invalid Unicode property!')
 
-        except StopIteration:
+        except StopIteration as e:
             if brackets:
-                raise SyntaxError("Missing or unmatched ':]' at {}!".format(index))
+                raise SyntaxError(f"Missing or unmatched ':]' at {index}!") from e
             else:
-                raise SyntaxError("Missing or unmatched '{{' at {}!".format(index))
+                raise SyntaxError(f"Missing or unmatched '{{' at {index}!") from e
 
-        return ''.join(prop).lower(), ''.join(value).lower()
+        p = ''.join(prop).lower()
+        v = ''.join(value).lower()
+
+        # Ensure when using POSIX form, that any property considered a compatibility property uses the POSIX form.
+        # POSIX form is not guaranteed to be different from standard form and sometimes is just an alias for standard.
+        if brackets and p in _COMPATIBILITY_PROPERTIES:
+            p = 'posix' + p
+
+        return p, v
 
     def get_named_unicode(self, i: _util.StringIter) -> str:
         """Get Unicode name."""
@@ -262,13 +282,13 @@ class _SearchParser(Generic[AnyStr]):
         value = []
         try:
             if next(i) != '{':
-                raise ValueError("Named Unicode missing '{{' {}!".format(i.index - 1))
+                raise ValueError(f"Named Unicode missing '{{' {i.index - 1}!")
             c = next(i)
             while c != '}':
                 value.append(c)
                 c = next(i)
-        except Exception:
-            raise SyntaxError("Unmatched '{{' at {}!".format(index))
+        except Exception as e:
+            raise SyntaxError(f"Unmatched '{{' at {index}!") from e
 
         return ''.join(value)
 
@@ -288,8 +308,10 @@ class _SearchParser(Generic[AnyStr]):
             mark = self.unicode_props("m", None, in_group=False)[0]
             current.extend(self._grapheme_cluster.format(no_mark, mark, mark))
         elif t == "e":
+            _util.warn_deprecated(R"The \e reference has been deprecated, please use \x1b instead")
             current.append(self._re_escape)
         elif t == "h":
+            _util.warn_deprecated(R"The \h reference has been deprecated, please use \p{Horiz_Space} instead")
             current.extend(self.unicode_props('blank', None, in_group=in_group))
             if in_group:
                 self.found_property = True
@@ -312,7 +334,7 @@ class _SearchParser(Generic[AnyStr]):
             current.extend(["\\", t])
         return current
 
-    def get_comments(self, i: _util.StringIter) -> Optional[str]:
+    def get_comments(self, i: _util.StringIter) -> str | None:
         """Get comments."""
 
         index = i.index
@@ -338,56 +360,61 @@ class _SearchParser(Generic[AnyStr]):
                 value.append(c)
                 c = next(i)
             value.append(c)
-        except StopIteration:
-            raise SyntaxError("Unmatched '(' at {}!".format(index - 1))
+        except StopIteration as e:
+            raise SyntaxError(f"Unmatched '(' at {index - 1}!") from e
 
         return ''.join(value)
 
-    def get_flags(self, i: _util.StringIter, scoped: bool = False) -> Optional[str]:
-        """Get flags."""
+    def get_flags(self, i: _util.StringIter) -> tuple[str | None, bool]:
+        """
+        Get flags.
+
+        In Re, flags are quite predictable when global or scoped.
+        Global can never be disabled with minus, and never have a `:` after them.
+        The global flag set is also very specific, but can be used as enablers in scoped.
+
+        The returned scoped status will indicate whether flags are generally considered
+        scoped flags or global flags.
+        """
 
         index = i.index
         value = ['(']
         toggle = False
-        end = ':' if scoped else ')'
+        smells_scoped = False
         try:
             c = next(i)
             if c != '?':
                 i.rewind(1)
-                return None
+                return None, False
             value.append(c)
             c = next(i)
-            while c != end:
+            while c not in _SCOPED_END:
                 if toggle:
                     if c not in _SCOPED_FLAGS:
                         raise ValueError('Bad scope')
-                    toggle = False
-                elif scoped and c == '-':
+                elif c == '-':
+                    smells_scoped = True
                     toggle = True
-                elif scoped and c in _GLOBAL_FLAGS:
-                    raise ValueError("Bad flag")
-                elif c not in _GLOBAL_FLAGS and c not in _SCOPED_FLAGS:
+                elif c not in _GLOBAL_FLAGS:
                     raise ValueError("Bad flag")
                 value.append(c)
                 c = next(i)
+            if smells_scoped and c != ':':
+                raise ValueError("Bad flag")
+            elif c == ':':
+                smells_scoped = True
             value.append(c)
         except Exception:
             i.rewind(i.index - index)
             value = []
 
-        return ''.join(value) if value else None
+        return ''.join(value) if value else None, smells_scoped
 
     def subgroup(self, t: str, i: _util.StringIter) -> list[str]:
         """Handle parenthesis."""
 
         current = []  # type: list[str]
 
-        # (?flags)
-        flags = self.get_flags(i)
-        if flags:
-            self.flags(flags[2:-1])
-            return [flags]
-
         # (?#comment)
         comments = self.get_comments(i)
         if comments:
@@ -396,11 +423,13 @@ class _SearchParser(Generic[AnyStr]):
         verbose = self.verbose
         unicode_flag = self.unicode
 
-        # (?flags:pattern)
-        flags = self.get_flags(i, True)
+        # (?flags:pattern) or (?flags)
+        flags, scoped = self.get_flags(i)
         if flags:  # pragma: no cover
             t = flags
-            self.flags(flags[2:-1], scoped=True)
+            self.flags(flags[2:-1], scoped=scoped)
+            if not scoped:
+                return [flags]
 
         current = []
         try:
@@ -464,7 +493,7 @@ class _SearchParser(Generic[AnyStr]):
                 elif t == "[":
                     index = i.index
                     try:
-                        prop = self.get_unicode_property(i, True)  # type: Optional[tuple[str, str]]
+                        prop = self.get_unicode_property(i, True)  # type: tuple[str, str] | None
                     except Exception:
                         prop = None
                         i.rewind(i.index - index)
@@ -503,9 +532,9 @@ class _SearchParser(Generic[AnyStr]):
             if temp == '[]':
                 # We specified some properties, but they are all
                 # out of reach.  Therefore we can match nothing.
-                current = ['[^{}]'.format(_uniprops.ASCII_RANGE if self.is_bytes else _uniprops.UNICODE_RANGE)]
+                current = [f'[^{_uniprops.ASCII_RANGE if self.is_bytes else _uniprops.UNICODE_RANGE}]']
             elif temp == '[^]':
-                current = ['[{}]'.format(_uniprops.ASCII_RANGE if self.is_bytes else _uniprops.UNICODE_RANGE)]
+                current = [f'[{_uniprops.ASCII_RANGE if self.is_bytes else _uniprops.UNICODE_RANGE}]']
             else:
                 current = [temp]
 
@@ -538,15 +567,15 @@ class _SearchParser(Generic[AnyStr]):
         value = ord(_unicodedata.lookup(name))
         if self.is_bytes and value > 0xFF:
             if not in_group:
-                return ['[^{}]'.format(_uniprops.ASCII_RANGE if self.is_bytes else _uniprops.UNICODE_RANGE)]
+                return [f'[^{_uniprops.ASCII_RANGE if self.is_bytes else _uniprops.UNICODE_RANGE}]']
             else:
                 return ['']
-        return ['\\{:03o}'.format(value) if value <= 0xFF else chr(value)]
+        return [f'\\{value:03o}' if value <= 0xFF else chr(value)]
 
     def unicode_props(
         self,
         props: str,
-        prop_value: Optional[str],
+        prop_value: str | None,
         in_group: bool = False,
         negate: bool = False
     ) -> list[str]:
@@ -575,8 +604,8 @@ class _SearchParser(Generic[AnyStr]):
         v = _uniprops.get_unicode_property(props, prop_value, mode)
         if not in_group:
             if not v:
-                v = '^{}'.format(_uniprops.ASCII_RANGE if self.is_bytes else _uniprops.UNICODE_RANGE)
-            v = "[{}]".format(v)
+                v = f'^{_uniprops.ASCII_RANGE if self.is_bytes else _uniprops.UNICODE_RANGE}'
+            v = f"[{v}]"
         properties = [v]
 
         return properties
@@ -585,12 +614,12 @@ class _SearchParser(Generic[AnyStr]):
         """The main group: group 0."""
 
         current = []
-        while True:
-            try:
+        try:
+            while True:
                 t = next(i)
                 current.extend(self.normal(t, i))
-            except StopIteration:
-                break
+        except StopIteration:
+            pass
         return current
 
     def _parse(self, search: str) -> str:
@@ -618,17 +647,17 @@ class _SearchParser(Generic[AnyStr]):
             retry = False
             try:
                 new_pattern = self.main_group(i)
-            except GlobalRetryException:
+            except GlobalRetryException as e:
                 # Prevent a loop of retry over and over for a pattern like ((?u)(?a))
                 # or (?-x:(?x))
                 if self.temp_global_flag_swap['unicode']:
                     if self.global_flag_swap['unicode']:
-                        raise LoopException('Global unicode flag recursion.')
+                        raise LoopException('Global unicode flag recursion.') from e
                     else:
                         self.global_flag_swap["unicode"] = True
                 if self.temp_global_flag_swap['verbose']:
                     if self.global_flag_swap['verbose']:
-                        raise LoopException('Global verbose flag recursion.')
+                        raise LoopException('Global verbose flag recursion.') from e
                     else:
                         self.global_flag_swap['verbose'] = True
                 self.temp_global_flag_swap = {
@@ -659,12 +688,12 @@ class _ReplaceParser(Generic[AnyStr]):
         self._template = template  # type: AnyStr
         self.use_format = use_format
         self.end_found = False
-        self.group_slots = []  # type: list[tuple[int, tuple[Optional[int], Optional[int], Any]]]
+        self.group_slots = []  # type: list[tuple[int, tuple[int | None, int | None, Any]]]
         self.literal_slots = []  # type: list[str]
         self.result = []  # type: list[str]
         self.span_stack = []  # type: list[int]
         self.single_stack = []  # type: list[int]
-        self.literals = []  # type: list[Optional[AnyStr]]
+        self.literals = []  # type: list[AnyStr | None]
         self.groups = []  # type: list[tuple[int, int]]
         self.slot = 0
         self.manual = False
@@ -741,8 +770,8 @@ class _ReplaceParser(Generic[AnyStr]):
                             while c != ']':
                                 findex.append(c)
                                 c = self.format_next(i)
-                        except StopIteration:
-                            raise SyntaxError("Unmatched '[' at {}".format(sindex - 1))
+                        except StopIteration as e:
+                            raise SyntaxError(f"Unmatched '[' at {sindex - 1}") from e
                         idx = self.parse_format_index(''.join(findex))
                         value.append((_util.FMT_INDEX, idx))
                         c = self.format_next(i)
@@ -758,13 +787,13 @@ class _ReplaceParser(Generic[AnyStr]):
                 if c == '!':
                     c = self.format_next(i)
                     if c not in _FMT_CONV_TYPE:
-                        raise SyntaxError("Invalid conversion type at {}!".format(i.index - 1))
+                        raise SyntaxError(f"Invalid conversion type at {i.index - 1}!")
                     value.append((_util.FMT_CONV, c))
                     c = self.format_next(i)
 
                 # Format spec
                 if c == ':':
-                    fill = None  # type: Optional[str]
+                    fill = None  # type: str | None
                     width = []
                     align = None
                     convert = None
@@ -796,7 +825,7 @@ class _ReplaceParser(Generic[AnyStr]):
                             fill = None
                         if fill is not None:
                             if c not in ('<', '>', '^'):
-                                raise SyntaxError('Invalid format spec char at {}!'.format(i.index - 1))
+                                raise SyntaxError(f'Invalid format spec char at {i.index - 1}!')
                             align = c
                             c = self.format_next(i)
 
@@ -829,9 +858,9 @@ class _ReplaceParser(Generic[AnyStr]):
                     )
 
             if c != '}':
-                raise SyntaxError("Unmatched '{{' at {}".format(index - 1))
-        except StopIteration:
-            raise SyntaxError("Unmatched '{{' at {}!".format(index - 1))
+                raise SyntaxError(f"Unmatched '{{' at {index - 1}")
+        except StopIteration as e:
+            raise SyntaxError(f"Unmatched '{{' at {index - 1}!") from e
 
         return field, value
 
@@ -852,9 +881,9 @@ class _ReplaceParser(Generic[AnyStr]):
                 self.get_single_stack()
                 self.result.append(t)
             else:
-                raise SyntaxError("Unmatched '}}' at {}!".format(i.index - 2))
+                raise SyntaxError(f"Unmatched '}}' at {i.index - 2}!")
 
-    def get_octal(self, c: str, i: _util.StringIter) -> Optional[str]:
+    def get_octal(self, c: str, i: _util.StringIter) -> str | None:
         """Get octal."""
 
         index = i.index
@@ -862,14 +891,14 @@ class _ReplaceParser(Generic[AnyStr]):
         zero_count = 0
         try:
             if c == '0':
-                for x in range(3):
+                for _ in range(3):
                     if c != '0':
                         break
                     value.append(c)
                     c = next(i)
             zero_count = len(value)
             if zero_count < 3:
-                for x in range(3 - zero_count):
+                for _ in range(3 - zero_count):
                     if c not in _OCTAL:
                         break
                     value.append(c)
@@ -902,7 +931,7 @@ class _ReplaceParser(Generic[AnyStr]):
             if self.use_format and value in _CURLY_BRACKETS_ORD:
                 self.handle_format(chr(value), i)
             elif value <= 0xFF:
-                self.result.append('\\{:03o}'.format(value))
+                self.result.append(f'\\{value:03o}')
             else:
                 self.result.append(chr(value))
 
@@ -913,13 +942,13 @@ class _ReplaceParser(Generic[AnyStr]):
         value = []
         try:
             if next(i) != '{':
-                raise SyntaxError("Named Unicode missing '{{' at {}!".format(i.index - 1))
+                raise SyntaxError(f"Named Unicode missing '{{' at {i.index - 1}!")
             c = next(i)
             while c != '}':
                 value.append(c)
                 c = next(i)
-        except StopIteration:
-            raise SyntaxError("Unmatched '}}' at {}!".format(index))
+        except StopIteration as e:
+            raise SyntaxError(f"Unmatched '}}' at {index}!") from e
 
         return ''.join(value)
 
@@ -936,7 +965,7 @@ class _ReplaceParser(Generic[AnyStr]):
         if self.use_format and value in _CURLY_BRACKETS_ORD:
             self.handle_format(chr(value), i)
         elif value <= 0xFF:
-            self.result.append('\\{:03o}'.format(value))
+            self.result.append(f'\\{value:03o}')
         else:
             self.result.append(chr(value))
 
@@ -944,37 +973,37 @@ class _ReplaceParser(Generic[AnyStr]):
         """Get narrow Unicode."""
 
         value = []
-        for x in range(3):
+        for _ in range(3):
             c = next(i)
             if c == '0':
                 value.append(c)
             else:  # pragma: no cover
-                raise SyntaxError('Invalid wide Unicode character at {}!'.format(i.index - 1))
+                raise SyntaxError(f'Invalid wide Unicode character at {i.index - 1}!')
 
         c = next(i)
         if c in ('0', '1'):
             value.append(c)
         else:  # pragma: no cover
-            raise SyntaxError('Invalid wide Unicode character at {}!'.format(i.index - 1))
+            raise SyntaxError(f'Invalid wide Unicode character at {i.index - 1}!')
 
-        for x in range(4):
+        for _ in range(4):
             c = next(i)
             if c.lower() in _HEX:
                 value.append(c)
             else:  # pragma: no cover
-                raise SyntaxError('Invalid wide Unicode character at {}!'.format(i.index - 1))
+                raise SyntaxError(f'Invalid wide Unicode character at {i.index - 1}!')
         return ''.join(value)
 
     def get_narrow_unicode(self, i: _util.StringIter) -> str:
         """Get narrow Unicode."""
 
         value = []
-        for x in range(4):
+        for _ in range(4):
             c = next(i)
             if c.lower() in _HEX:
                 value.append(c)
             else:  # pragma: no cover
-                raise SyntaxError('Invalid Unicode character at {}!'.format(i.index - 1))
+                raise SyntaxError(f'Invalid Unicode character at {i.index - 1}!')
         return ''.join(value)
 
     def parse_unicode(self, i: _util.StringIter, wide: bool = False) -> None:
@@ -991,7 +1020,7 @@ class _ReplaceParser(Generic[AnyStr]):
         if self.use_format and value in _CURLY_BRACKETS_ORD:
             self.handle_format(chr(value), i)
         elif value <= 0xFF:
-            self.result.append('\\{:03o}'.format(value))
+            self.result.append(f'\\{value:03o}')
         else:
             self.result.append(chr(value))
 
@@ -999,12 +1028,12 @@ class _ReplaceParser(Generic[AnyStr]):
         """Get byte."""
 
         value = []
-        for x in range(2):
+        for _x in range(2):
             c = next(i)
             if c.lower() in _HEX:
                 value.append(c)
             else:  # pragma: no cover
-                raise SyntaxError('Invalid byte character at {}!'.format(i.index - 1))
+                raise SyntaxError(f'Invalid byte character at {i.index - 1}!')
         return ''.join(value)
 
     def parse_bytes(self, i: _util.StringIter) -> None:
@@ -1020,7 +1049,7 @@ class _ReplaceParser(Generic[AnyStr]):
         if self.use_format and value in _CURLY_BRACKETS_ORD:
             self.handle_format(chr(value), i)
         else:
-            self.result.append('\\{:03o}'.format(value))
+            self.result.append(f'\\{value:03o}')
 
     def get_named_group(self, t: str, i: _util.StringIter) -> str:
         """Get group number."""
@@ -1030,7 +1059,7 @@ class _ReplaceParser(Generic[AnyStr]):
         try:
             c = next(i)
             if c != "<":
-                raise SyntaxError("Group missing '<' at {}!".format(i.index - 1))
+                raise SyntaxError(f"Group missing '<' at {i.index - 1}!")
             value.append(c)
             c = next(i)
             if c in _DIGIT:
@@ -1050,13 +1079,13 @@ class _ReplaceParser(Generic[AnyStr]):
                     c = next(i)
                 value.append(c)
             else:
-                raise SyntaxError("Invalid group character at {}!".format(i.index - 1))
-        except StopIteration:
-            raise SyntaxError("Unmatched '<' at {}!".format(index))
+                raise SyntaxError(f"Invalid group character at {i.index - 1}!")
+        except StopIteration as e:
+            raise SyntaxError(f"Unmatched '<' at {index}!") from e
 
         return ''.join(value)
 
-    def get_group(self, t: str, i: _util.StringIter) -> Optional[str]:
+    def get_group(self, t: str, i: _util.StringIter) -> str | None:
         """Get group number."""
 
         value = []
@@ -1154,8 +1183,8 @@ class _ReplaceParser(Generic[AnyStr]):
 
         i = _util.StringIter(template)
 
-        while True:
-            try:
+        try:
+            while True:
                 t = next(i)
                 if self.use_format and t in _CURLY_BRACKETS:
                     self.handle_format(t, i)
@@ -1168,9 +1197,8 @@ class _ReplaceParser(Generic[AnyStr]):
                         raise
                 else:
                     self.result.append(t)
-
-            except StopIteration:
-                break
+        except StopIteration:
+            pass
 
         if len(self.result) > 1:
             self.literal_slots.append("".join(self.result))
@@ -1188,7 +1216,19 @@ class _ReplaceParser(Generic[AnyStr]):
         else:
             self._template = self._parse_template(self._original)
 
-        self.groups, self.literals = _parser.parse_template(self._template, self.pattern)
+        if _util.PY312:
+            count = 0
+            for part in _parser.parse_template(self._template, self.pattern):
+                if isinstance(part, int):
+                    self.groups.append((count, part))
+                    self.literals.append(None)
+                elif part:
+                    self.literals.append(cast(AnyStr, part))
+                else:
+                    continue
+                count += 1
+        else:
+            self.groups, self.literals = _parser.parse_template(self._template, self.pattern)
 
     def span_case(self, i: _util.StringIter, case: int) -> None:
         """Uppercase or lowercase the next range of characters until end marker is found."""
@@ -1262,7 +1302,7 @@ class _ReplaceParser(Generic[AnyStr]):
         except StopIteration:
             pass
 
-    def get_single_stack(self) -> Optional[int]:
+    def get_single_stack(self) -> int | None:
         """Get the correct single stack item to use."""
 
         single = None
@@ -1296,7 +1336,7 @@ class _ReplaceParser(Generic[AnyStr]):
     def handle_group(
         self,
         text: str,
-        capture: Optional[tuple[tuple[int, Any], ...]] = None,
+        capture: tuple[tuple[int, Any], ...] | None = None,
         is_format: bool = False
     ) -> None:
         """Handle groups."""
@@ -1321,7 +1361,7 @@ class _ReplaceParser(Generic[AnyStr]):
                 (
                     (self.span_stack[-1] if self.span_stack else None),
                     self.get_single_stack(),
-                    (tuple() if self.is_bytes else '') if capture is None else capture
+                    (() if self.is_bytes else '') if capture is None else capture
                 )
             )
         )
@@ -1332,7 +1372,7 @@ class _ReplaceParser(Generic[AnyStr]):
 
         return self._original
 
-    def parse(self) -> 'ReplaceTemplate[AnyStr]':
+    def parse(self) -> ReplaceTemplate[AnyStr]:
         """Parse template."""
 
         if not isinstance(self.pattern.pattern, type(self._original)):
@@ -1356,8 +1396,8 @@ class ReplaceTemplate(_util.Immutable, Generic[AnyStr]):
     __slots__ = ("groups", "group_slots", "literals", "pattern_hash", "use_format", "_hash", "_bytes")
 
     groups: tuple[tuple[int, int], ...]
-    group_slots: tuple[tuple[int, tuple[Optional[int], Optional[int], Any]], ...]
-    literals: tuple[Optional[AnyStr], ...]
+    group_slots: tuple[tuple[int, tuple[int | None, int | None, Any]], ...]
+    literals: tuple[AnyStr | None, ...]
     pattern_hash: int
     use_format: bool
     _hash: int
@@ -1366,8 +1406,8 @@ class ReplaceTemplate(_util.Immutable, Generic[AnyStr]):
     def __init__(
         self,
         groups: tuple[tuple[int, int], ...],
-        group_slots: tuple[tuple[int, tuple[Optional[int], Optional[int], Any]], ...],
-        literals: tuple[Optional[AnyStr], ...],
+        group_slots: tuple[tuple[int, tuple[int | None, int | None, Any]], ...],
+        literals: tuple[AnyStr | None, ...],
         pattern_hash: int,
         use_format: bool,
         is_bytes: bool
@@ -1390,7 +1430,7 @@ class ReplaceTemplate(_util.Immutable, Generic[AnyStr]):
             )
         )
 
-    def __call__(self, m: Optional[Match[AnyStr]]) -> AnyStr:
+    def __call__(self, m: Match[AnyStr] | None) -> AnyStr:
         """Call."""
 
         return self.expand(m)
@@ -1445,17 +1485,17 @@ class ReplaceTemplate(_util.Immutable, Generic[AnyStr]):
                 break
         return g_index
 
-    def _get_group_attributes(self, index: int) -> tuple[Optional[int], Optional[int], Any]:
+    def _get_group_attributes(self, index: int) -> tuple[int | None, int | None, Any]:
         """Find and return the appropriate group case."""
 
-        g_case = (None, None, -1)  # type: tuple[Optional[int], Optional[int], Any]
+        g_case = (None, None, -1)  # type: tuple[int | None, int | None, Any]
         for group in self.group_slots:
             if group[0] == index:
                 g_case = group[1]
                 break
         return g_case
 
-    def expand(self, m: Optional[Match[AnyStr]]) -> AnyStr:
+    def expand(self, m: Match[AnyStr] | None) -> AnyStr:
         """Using the template, expand the string."""
 
         if m is None:
@@ -1478,14 +1518,14 @@ class ReplaceTemplate(_util.Immutable, Generic[AnyStr]):
                         l = m.group(g_index)
                         if l is None:
                             l = sep
-                    except IndexError:  # pragma: no cover
-                        raise IndexError("'{}' is out of range!".format(g_index))
+                    except IndexError as e:  # pragma: no cover
+                        raise IndexError(f"'{g_index}' is out of range!") from e
                 else:
                     # String format replace
                     try:
                         obj = m.group(g_index)
-                    except IndexError:  # pragma: no cover
-                        raise IndexError("'{}' is out of range!".format(g_index))
+                    except IndexError as e:  # pragma: no cover
+                        raise IndexError(f"'{g_index}' is out of range!") from e
                     l = _util.format_captures(
                         [] if obj is None else [obj],
                         capture,
