@@ -16,11 +16,9 @@ Add the ability to use the following backrefs with re:
  - `\PL`, `\P{Lu}`, `\p{^Lu}`                                    - Inverse Unicode properties (search Unicode)
  - `\N{Black Club Suit}`                                         - Unicode character by name (search & replace)
  - `\u0000` and `\U00000000`                                     - Unicode characters (replace)
- - `\e`                                                          - Escape character (search)
  - `\m`                                                          - Starting word boundary (search)
  - `\M`                                                          - Ending word boundary (search)
  - `\R`                                                          - Generic line breaks (search)
- - `\h`                                                          - Horizontal whitespace (search)
  - `\X`                                                          - Simplified grapheme clusters (search)
 
 Licensed under MIT
@@ -33,7 +31,7 @@ from functools import lru_cache as _lru_cache
 from . import util as _util
 from . import _bre_parse
 from ._bre_parse import ReplaceTemplate
-from typing import AnyStr, Pattern, Match, Callable, Any, Optional, Generic, Mapping, Iterator, cast
+from typing import AnyStr, Pattern, Match, Callable, Any, Generic, Mapping, Iterator, cast
 
 __all__ = (
     "expand", "expandf", "search", "match", "fullmatch", "split", "findall", "finditer", "sub", "subf",
@@ -118,7 +116,7 @@ def _is_replace(obj: Any) -> bool:
 
 
 def _apply_replace_backrefs(
-    m: Optional[Match[AnyStr]],
+    m: Match[AnyStr] | None,
     repl: ReplaceTemplate[AnyStr] | AnyStr,
     flags: int = 0
 ) -> AnyStr:
@@ -217,10 +215,10 @@ class Bre(_util.Immutable, Generic[AnyStr]):
         return self._pattern.groupindex
 
     @property
-    def groups(self) -> tuple[Optional[AnyStr], ...]:
+    def groups(self) -> tuple[AnyStr | None, ...]:
         """Return groups."""
 
-        return cast('tuple[Optional[AnyStr], ...]', self._pattern.groups)
+        return cast('tuple[AnyStr | None, ...]', self._pattern.groups)
 
     @property
     def scanner(self) -> Any:
@@ -291,7 +289,7 @@ class Bre(_util.Immutable, Generic[AnyStr]):
         string: AnyStr,
         *args: Any,
         **kwargs: Any
-    ) -> Optional[Match[AnyStr]]:
+    ) -> Match[AnyStr] | None:
         """Apply `search`."""
 
         return self._pattern.search(string, *args, **kwargs)
@@ -301,7 +299,7 @@ class Bre(_util.Immutable, Generic[AnyStr]):
         string: AnyStr,
         *args: Any,
         **kwargs: Any
-    ) -> Optional[Match[AnyStr]]:
+    ) -> Match[AnyStr] | None:
         """Apply `match`."""
 
         return self._pattern.match(string, *args, **kwargs)
@@ -311,7 +309,7 @@ class Bre(_util.Immutable, Generic[AnyStr]):
         string: AnyStr,
         *args: Any,
         **kwargs: Any
-    ) -> Optional[Match[AnyStr]]:
+    ) -> Match[AnyStr] | None:
         """Apply `fullmatch`."""
 
         return self._pattern.fullmatch(string, *args, **kwargs)
@@ -394,8 +392,8 @@ class Bre(_util.Immutable, Generic[AnyStr]):
 def compile(  # noqa A001
     pattern: AnyStr | Pattern[AnyStr] | Bre[AnyStr],
     flags: int = 0,
-    auto_compile: Optional[bool] = None
-) -> 'Bre[AnyStr]':
+    auto_compile: bool | None = None
+) -> Bre[AnyStr]:
     """Compile both the search or search and replace into one object."""
 
     if isinstance(pattern, Bre):
@@ -453,14 +451,14 @@ def purge() -> None:
     _re.purge()
 
 
-def expand(m: Optional[Match[AnyStr]], repl: ReplaceTemplate[AnyStr] | AnyStr) -> AnyStr:
+def expand(m: Match[AnyStr] | None, repl: ReplaceTemplate[AnyStr] | AnyStr) -> AnyStr:
     """Expand the string using the replace pattern or function."""
 
     _assert_expandable(repl)
     return _apply_replace_backrefs(m, repl)
 
 
-def expandf(m: Optional[Match[AnyStr]], repl: ReplaceTemplate[AnyStr] | AnyStr) -> AnyStr:
+def expandf(m: Match[AnyStr] | None, repl: ReplaceTemplate[AnyStr] | AnyStr) -> AnyStr:
     """Expand the string using the format replace pattern or function."""
 
     _assert_expandable(repl, True)
@@ -470,85 +468,90 @@ def expandf(m: Optional[Match[AnyStr]], repl: ReplaceTemplate[AnyStr] | AnyStr)
 def search(
     pattern: AnyStr | Pattern[AnyStr] | Bre[AnyStr],
     string: AnyStr,
+    flags: int | _re.RegexFlag = 0,
     *args: Any,
     **kwargs: Any
-) -> Optional[Match[AnyStr]]:
+) -> Match[AnyStr] | None:
     """Apply `search` after applying backrefs."""
 
-    flags = args[2] if len(args) > 2 else kwargs.get('flags', 0)
-    return _re.search(_apply_search_backrefs(pattern, flags), string, *args, **kwargs)
+    return _re.search(_apply_search_backrefs(pattern, flags), string, flags, *args, **kwargs)
 
 
-def match(
+def prefixmatch(
     pattern: AnyStr | Pattern[AnyStr] | Bre[AnyStr],
     string: AnyStr,
-    *args: Any,
+    flags: int | _re.RegexFlag = 0,
     **kwargs: Any
-) -> Optional[Match[AnyStr]]:
+) -> Match[AnyStr] | None:
     """Apply `match` after applying backrefs."""
 
-    flags = args[2] if len(args) > 2 else kwargs.get('flags', 0)
-    return _re.match(_apply_search_backrefs(pattern, flags), string, *args, **kwargs)
+    return _re.match(_apply_search_backrefs(pattern, flags), string, flags=flags, **kwargs)
+
+
+match = prefixmatch
 
 
 def fullmatch(
     pattern: AnyStr | Pattern[AnyStr] | Bre[AnyStr],
     string: AnyStr,
-    *args: Any,
+    flags: int | _re.RegexFlag = 0,
     **kwargs: Any
-) -> Optional[Match[AnyStr]]:
+) -> Match[AnyStr] | None:
     """Apply `fullmatch` after applying backrefs."""
 
-    flags = args[2] if len(args) > 2 else kwargs.get('flags', 0)
-    return _re.fullmatch(_apply_search_backrefs(pattern, flags), string, *args, **kwargs)
+    return _re.fullmatch(_apply_search_backrefs(pattern, flags), string, flags=flags, **kwargs)
 
 
 def split(
     pattern: AnyStr | Pattern[AnyStr] | Bre[AnyStr],
     string: AnyStr,
-    *args: Any,
+    maxsplit: int = 0,
+    flags: int | _re.RegexFlag = 0,
     **kwargs: Any
 ) -> list[AnyStr]:
     """Apply `split` after applying backrefs."""
 
-    flags = args[3] if len(args) > 3 else kwargs.get('flags', 0)
-    return _re.split(_apply_search_backrefs(pattern, flags), string, *args, **kwargs)
+    return _re.split(
+        _apply_search_backrefs(pattern, flags),
+        string,
+        maxsplit=maxsplit,
+        flags=flags,
+        **kwargs
+    )
 
 
 def findall(
     pattern: AnyStr | Pattern[AnyStr] | Bre[AnyStr],
     string: AnyStr,
-    *args: Any,
+    flags: int | _re.RegexFlag = 0,
     **kwargs: Any
 ) -> list[AnyStr] | list[tuple[AnyStr, ...]]:
     """Apply `findall` after applying backrefs."""
 
-    flags = args[2] if len(args) > 2 else kwargs.get('flags', 0)
-    return _re.findall(_apply_search_backrefs(pattern, flags), string, *args, **kwargs)
+    return _re.findall(_apply_search_backrefs(pattern, flags), string, flags=flags, **kwargs)
 
 
 def finditer(
     pattern: AnyStr | Pattern[AnyStr] | Bre[AnyStr],
     string: AnyStr,
-    *args: Any,
+    flags: int | _re.RegexFlag = 0,
     **kwargs: Any
 ) -> Iterator[Match[AnyStr]]:
     """Apply `finditer` after applying backrefs."""
 
-    flags = args[2] if len(args) > 2 else kwargs.get('flags', 0)
-    return _re.finditer(_apply_search_backrefs(pattern, flags), string, *args, **kwargs)
+    return _re.finditer(_apply_search_backrefs(pattern, flags), string, flags=flags, **kwargs)
 
 
 def sub(
     pattern: AnyStr | Pattern[AnyStr] | Bre[AnyStr],
     repl: AnyStr | Callable[..., AnyStr],
     string: AnyStr,
-    *args: Any,
+    count: int = 0,
+    flags: int | _re.RegexFlag = 0,
     **kwargs: Any
 ) -> AnyStr:
     """Apply `sub` after applying backrefs."""
 
-    flags = args[4] if len(args) > 4 else kwargs.get('flags', 0)
     is_replace = _is_replace(repl)
     is_string = isinstance(repl, (str, bytes))
     if is_replace and cast(ReplaceTemplate[AnyStr], repl).use_format:
@@ -556,7 +559,12 @@ def sub(
 
     pattern = compile_search(pattern, flags)
     return _re.sub(
-        pattern, (compile_replace(pattern, repl) if is_replace or is_string else repl), string, *args, **kwargs
+        pattern,
+        (compile_replace(pattern, repl) if is_replace or is_string else repl),
+        string,
+        count=count,
+        flags=0,
+        **kwargs
     )
 
 
@@ -564,12 +572,12 @@ def subf(
     pattern: AnyStr | Pattern[AnyStr] | Bre[AnyStr],
     repl: AnyStr | Callable[..., AnyStr],
     string: AnyStr,
-    *args: Any,
+    count: int = 0,
+    flags: int | _re.RegexFlag = 0,
     **kwargs: Any
 ) -> AnyStr:
     """Apply `sub` with format style replace."""
 
-    flags = args[4] if len(args) > 4 else kwargs.get('flags', 0)
     is_replace = _is_replace(repl)
     is_string = isinstance(repl, (str, bytes))
     if is_replace and not cast(ReplaceTemplate[AnyStr], repl).use_format:
@@ -578,8 +586,12 @@ def subf(
     pattern = compile_search(pattern, flags)
     rflags = FORMAT if is_string else 0
     return _re.sub(
-        pattern, (compile_replace(pattern, repl, flags=rflags) if is_replace or is_string else repl), string,
-        *args, **kwargs
+        pattern,
+        (compile_replace(pattern, repl, flags=rflags) if is_replace or is_string else repl),
+        string,
+        count=count,
+        flags=0,
+        **kwargs
     )
 
 
@@ -587,12 +599,12 @@ def subn(
     pattern: AnyStr | Pattern[AnyStr] | Bre[AnyStr],
     repl: AnyStr | Callable[..., AnyStr],
     string: AnyStr,
-    *args: Any,
+    count: int = 0,
+    flags: int | _re.RegexFlag = 0,
     **kwargs: Any
 ) -> tuple[AnyStr, int]:
     """Apply `subn` with format style replace."""
 
-    flags = args[4] if len(args) > 4 else kwargs.get('flags', 0)
     is_replace = _is_replace(repl)
     is_string = isinstance(repl, (str, bytes))
     if is_replace and cast(ReplaceTemplate[AnyStr], repl).use_format:
@@ -600,7 +612,12 @@ def subn(
 
     pattern = compile_search(pattern, flags)
     return _re.subn(
-        pattern, (compile_replace(pattern, repl) if is_replace or is_string else repl), string, *args, **kwargs
+        pattern,
+        (compile_replace(pattern, repl) if is_replace or is_string else repl),
+        string,
+        count=count,
+        flags=0,
+        **kwargs
     )
 
 
@@ -608,12 +625,12 @@ def subfn(
     pattern: AnyStr | Pattern[AnyStr] | Bre[AnyStr],
     repl: AnyStr | Callable[..., AnyStr],
     string: AnyStr,
-    *args: Any,
+    count: int = 0,
+    flags: int | _re.RegexFlag = 0,
     **kwargs: Any
 ) -> tuple[AnyStr, int]:
     """Apply `subn` after applying backrefs."""
 
-    flags = args[4] if len(args) > 4 else kwargs.get('flags', 0)
     is_replace = _is_replace(repl)
     is_string = isinstance(repl, (str, bytes))
     if is_replace and not cast(ReplaceTemplate[AnyStr], repl).use_format:
@@ -622,8 +639,12 @@ def subfn(
     pattern = compile_search(pattern, flags)
     rflags = FORMAT if is_string else 0
     return _re.subn(
-        pattern, (compile_replace(pattern, repl, flags=rflags) if is_replace or is_string else repl), string,
-        *args, **kwargs
+        pattern,
+        (compile_replace(pattern, repl, flags=rflags) if is_replace or is_string else repl),
+        string,
+        count=count,
+        flags=0,
+        **kwargs
     )
 
 
