@@ -9,7 +9,6 @@ Add the ability to use the following backrefs with re:
  - `\N{Black Club Suit}`                                        - Unicode character by name (replace)
  - `\u0000` and `\U00000000`                                    - Unicode characters (replace)
  - `\R`                                                         - Generic line breaks (search)
- - `\e`                                                         - Escape character (search)
 
 Licensed under MIT
 Copyright (c) 2015 - 2020 Isaac Muse <isaacmuse@gmail.com>
@@ -21,7 +20,7 @@ from functools import lru_cache as _lru_cache
 from . import util as _util
 from . import _bregex_parse
 from ._bregex_parse import ReplaceTemplate
-from typing import AnyStr, Callable, Any, Optional, Generic, Mapping, Iterator, cast
+from typing import AnyStr, Callable, Any, Generic, Mapping, Iterator, cast
 from ._bregex_typing import Pattern, Match
 
 __all__ = (
@@ -131,7 +130,7 @@ def _is_replace(obj: Any) -> bool:
 
 
 def _apply_replace_backrefs(
-    m: Optional[Match[AnyStr]],
+    m: Match[AnyStr] | None,
     repl: ReplaceTemplate[AnyStr] | AnyStr,
     flags: int = 0
 ) -> AnyStr:
@@ -231,10 +230,10 @@ class Bregex(_util.Immutable, Generic[AnyStr]):
         return cast(Mapping[str, int], self._pattern.groupindex)
 
     @property
-    def groups(self) -> tuple[Optional[AnyStr], ...]:
+    def groups(self) -> tuple[AnyStr | None, ...]:
         """Return groups."""
 
-        return cast('tuple[Optional[AnyStr], ...]', self._pattern.groups)
+        return cast('tuple[AnyStr | None, ...]', self._pattern.groups)
 
     @property
     def scanner(self) -> Any:
@@ -311,7 +310,7 @@ class Bregex(_util.Immutable, Generic[AnyStr]):
         string: AnyStr,
         *args: Any,
         **kwargs: Any
-    ) -> Optional[Match[AnyStr]]:
+    ) -> Match[AnyStr] | None:
         """Apply `search`."""
 
         return self._pattern.search(string, *args, **kwargs)
@@ -321,20 +320,20 @@ class Bregex(_util.Immutable, Generic[AnyStr]):
         string: AnyStr,
         *args: Any,
         **kwargs: Any
-    ) -> Optional[Match[AnyStr]]:
+    ) -> Match[AnyStr] | None:
         """Apply `match`."""
 
-        return cast(Optional[Match[AnyStr]], self._pattern.match(string, *args, **kwargs))
+        return cast('Match[AnyStr] | None', self._pattern.match(string, *args, **kwargs))
 
     def fullmatch(
         self,
         string: AnyStr,
         *args: Any,
         **kwargs: Any
-    ) -> Optional[Match[AnyStr]]:
+    ) -> Match[AnyStr] | None:
         """Apply `fullmatch`."""
 
-        return cast(Optional[Match[AnyStr]], self._pattern.fullmatch(string, *args, **kwargs))
+        return cast('Match[AnyStr] | None', self._pattern.fullmatch(string, *args, **kwargs))
 
     def split(
         self,
@@ -424,9 +423,9 @@ class Bregex(_util.Immutable, Generic[AnyStr]):
 def compile(  # noqa A001
     pattern: AnyStr | Pattern[AnyStr] | Bregex[AnyStr],
     flags: int = 0,
-    auto_compile: Optional[bool] = None,
+    auto_compile: bool | None = None,
     **kwargs: Any
-) -> 'Bregex[AnyStr]':
+) -> Bregex[AnyStr]:
     """Compile both the search or search and replace into one object."""
 
     if isinstance(pattern, Bregex):
@@ -485,62 +484,65 @@ def purge() -> None:
     _regex.purge()
 
 
-def expand(m: Optional[Match[AnyStr]], repl: ReplaceTemplate[AnyStr] | AnyStr) -> AnyStr:
+def expand(m: Match[AnyStr] | None, repl: ReplaceTemplate[AnyStr] | AnyStr) -> AnyStr:
     """Expand the string using the replace pattern or function."""
 
     _assert_expandable(repl)
     return _apply_replace_backrefs(m, repl)
 
 
-def expandf(m: Optional[Match[AnyStr]], repl: ReplaceTemplate[AnyStr] | AnyStr) -> AnyStr:
+def expandf(m: Match[AnyStr] | None, repl: ReplaceTemplate[AnyStr] | AnyStr) -> AnyStr:
     """Expand the string using the format replace pattern or function."""
 
     _assert_expandable(repl, True)
     return _apply_replace_backrefs(m, repl, flags=FORMAT)
 
 
-def match(
+def prefixmatch(
     pattern: AnyStr | Pattern[AnyStr] | Bregex[AnyStr],
     string: AnyStr,
+    flags: int = 0,
     *args: Any,
     **kwargs: Any
-) -> Optional[Match[AnyStr]]:
+) -> Match[AnyStr] | None:
     """Wrapper for `match`."""
 
-    flags = args[2] if len(args) > 2 else kwargs.get('flags', 0)
     return cast(
-        Optional[Match[AnyStr]],
-        _regex.match(_apply_search_backrefs(pattern, flags), string, *args, **kwargs)
+        'Match[AnyStr] | None',
+        _regex.match(_apply_search_backrefs(pattern, flags), string, flags, *args, **kwargs)
     )
 
 
+match = prefixmatch
+
+
 def fullmatch(
     pattern: AnyStr | Pattern[AnyStr] | Bregex[AnyStr],
     string: AnyStr,
+    flags: int = 0,
     *args: Any,
     **kwargs: Any
-) -> Optional[Match[AnyStr]]:
+) -> Match[AnyStr] | None:
     """Wrapper for `fullmatch`."""
 
-    flags = args[2] if len(args) > 2 else kwargs.get('flags', 0)
     return cast(
-        Optional[Match[AnyStr]],
-        _regex.fullmatch(_apply_search_backrefs(pattern, flags), string, *args, **kwargs)
+        'Match[AnyStr] | None',
+        _regex.fullmatch(_apply_search_backrefs(pattern, flags), string, flags, *args, **kwargs)
     )
 
 
 def search(
     pattern: AnyStr | Pattern[AnyStr] | Bregex[AnyStr],
     string: AnyStr,
+    flags: int = 0,
     *args: Any,
     **kwargs: Any
-) -> Optional[Match[AnyStr]]:
+) -> Match[AnyStr] | None:
     """Wrapper for `search`."""
 
-    flags = args[2] if len(args) > 2 else kwargs.get('flags', 0)
     return cast(
-        Optional[Match[AnyStr]],
-        _regex.search(_apply_search_backrefs(pattern, flags), string, *args, **kwargs)
+        'Match[AnyStr] | None',
+        _regex.search(_apply_search_backrefs(pattern, flags), string, flags, *args, **kwargs)
     )
 
 
@@ -548,12 +550,13 @@ def sub(
     pattern: AnyStr | Pattern[AnyStr] | Bregex[AnyStr],
     repl: AnyStr | Callable[..., AnyStr],
     string: AnyStr,
+    count: int = 0,
+    flags: int = 0,
     *args: Any,
     **kwargs: Any
 ) -> AnyStr:
     """Wrapper for `sub`."""
 
-    flags = args[4] if len(args) > 4 else kwargs.get('flags', 0)
     is_replace = _is_replace(repl)
     is_string = isinstance(repl, (str, bytes))
     if is_replace and cast(ReplaceTemplate[AnyStr], repl).use_format:
@@ -564,6 +567,8 @@ def sub(
         AnyStr,
         _regex.sub(
             pattern, (compile_replace(pattern, repl) if is_replace or is_string else repl), string,
+            count,
+            0,
             *args, **kwargs
         )
     )
@@ -573,12 +578,13 @@ def subf(
     pattern: AnyStr | Pattern[AnyStr] | Bregex[AnyStr],
     repl: AnyStr | Callable[..., AnyStr],
     string: AnyStr,
+    count: int = 0,
+    flags: int = 0,
     *args: Any,
     **kwargs: Any
 ) -> AnyStr:
     """Wrapper for `subf`."""
 
-    flags = args[4] if len(args) > 4 else kwargs.get('flags', 0)
     is_replace = _is_replace(repl)
     is_string = isinstance(repl, (str, bytes))
     if is_replace and not cast(ReplaceTemplate[AnyStr], repl).use_format:
@@ -590,6 +596,8 @@ def subf(
         AnyStr,
         _regex.sub(
             pattern, (compile_replace(pattern, repl, flags=rflags) if is_replace or is_string else repl), string,
+            count,
+            0,
             *args, **kwargs
         )
     )
@@ -599,12 +607,13 @@ def subn(
     pattern: AnyStr | Pattern[AnyStr] | Bregex[AnyStr],
     repl: AnyStr | Callable[..., AnyStr],
     string: AnyStr,
+    count: int = 0,
+    flags: int = 0,
     *args: Any,
     **kwargs: Any
 ) -> tuple[AnyStr, int]:
     """Wrapper for `subn`."""
 
-    flags = args[4] if len(args) > 4 else kwargs.get('flags', 0)
     is_replace = _is_replace(repl)
     is_string = isinstance(repl, (str, bytes))
     if is_replace and cast(ReplaceTemplate[AnyStr], repl).use_format:
@@ -615,6 +624,8 @@ def subn(
         'tuple[AnyStr, int]',
         _regex.subn(
             pattern, (compile_replace(pattern, repl) if is_replace or is_string else repl), string,
+            count,
+            0,
             *args, **kwargs
         )
     )
@@ -624,12 +635,13 @@ def subfn(
     pattern: AnyStr | Pattern[AnyStr] | Bregex[AnyStr],
     repl: AnyStr | Callable[..., AnyStr],
     string: AnyStr,
+    count: int = 0,
+    flags: int = 0,
     *args: Any,
     **kwargs: Any
 ) -> tuple[AnyStr, int]:
     """Wrapper for `subfn`."""
 
-    flags = args[4] if len(args) > 4 else kwargs.get('flags', 0)
     is_replace = _is_replace(repl)
     is_string = isinstance(repl, (str, bytes))
     if is_replace and not cast(ReplaceTemplate[AnyStr], repl).use_format:
@@ -641,6 +653,8 @@ def subfn(
         'tuple[AnyStr, int]',
         _regex.subn(
             pattern, (compile_replace(pattern, repl, flags=rflags) if is_replace or is_string else repl), string,
+            count,
+            0,
             *args, **kwargs
         )
     )
@@ -649,57 +663,59 @@ def subfn(
 def split(
     pattern: AnyStr | Pattern[AnyStr] | Bregex[AnyStr],
     string: AnyStr,
+    maxsplit: int = 0,
+    flags: int = 0,
     *args: Any,
     **kwargs: Any
 ) -> list[AnyStr]:
     """Wrapper for `split`."""
 
-    flags = args[3] if len(args) > 3 else kwargs.get('flags', 0)
     return cast(
         'list[AnyStr]',
-        _regex.split(_apply_search_backrefs(pattern, flags), string, *args, **kwargs)
+        _regex.split(_apply_search_backrefs(pattern, flags), string, maxsplit, flags, *args, **kwargs)
     )
 
 
 def splititer(
     pattern: AnyStr | Pattern[AnyStr] | Bregex[AnyStr],
     string: AnyStr,
+    maxsplit: int = 0,
+    flags: int = 0,
     *args: Any,
     **kwargs: Any
 ) -> Iterator[AnyStr]:
     """Wrapper for `splititer`."""
 
-    flags = args[3] if len(args) > 3 else kwargs.get('flags', 0)
     return cast(
         Iterator[AnyStr],
-        _regex.splititer(_apply_search_backrefs(pattern, flags), string, *args, **kwargs)
+        _regex.splititer(_apply_search_backrefs(pattern, flags), string, maxsplit, flags, *args, **kwargs)
     )
 
 
 def findall(
     pattern: AnyStr | Pattern[AnyStr] | Bregex[AnyStr],
     string: AnyStr,
+    flags: int = 0,
     *args: Any,
     **kwargs: Any
 ) -> list[AnyStr] | list[tuple[AnyStr, ...]]:
     """Wrapper for `findall`."""
 
-    flags = args[2] if len(args) > 2 else kwargs.get('flags', 0)
     return cast(
         'list[AnyStr] | list[tuple[AnyStr, ...]]',
-        _regex.findall(_apply_search_backrefs(pattern, flags), string, *args, **kwargs)
+        _regex.findall(_apply_search_backrefs(pattern, flags), string, flags, *args, **kwargs)
     )
 
 
 def finditer(
     pattern: AnyStr | Pattern[AnyStr] | Bregex[AnyStr],
     string: AnyStr,
+    flags: int = 0,
     *args: Any,
     **kwargs: Any
 ) -> Iterator[Match[AnyStr]]:
     """Wrapper for `finditer`."""
 
-    flags = args[2] if len(args) > 2 else kwargs.get('flags', 0)
     return cast(
         Iterator[Match[AnyStr]],
         _regex.finditer(_apply_search_backrefs(pattern, flags), string, *args, **kwargs)
